#include <windows.h>
#include <iostream.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/stat.h>

// Read the size of the bitmap from the file header and decide whether to tile or centre it
// Centre if it's greater than a third of the desktop width and greater than a third of the desktop height
bool Calculate(const char *wallpaper)
{
	bool tile(true);

	HANDLE fhnd = CreateFile (wallpaper,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);

	if (fhnd != INVALID_HANDLE_VALUE)
	{
		BITMAPFILEHEADER fhdr;
		BITMAPINFOHEADER finfo;
		DWORD read;

		if (ReadFile(fhnd,&fhdr,sizeof(BITMAPFILEHEADER),&read,NULL)&&(read==sizeof(BITMAPFILEHEADER)))
		{
			if (ReadFile(fhnd,&finfo,sizeof(BITMAPINFOHEADER),&read,NULL)&&(read==sizeof(BITMAPINFOHEADER)))
			{
				int screenwidth = GetSystemMetrics(SM_CXSCREEN);
				int screenheight = GetSystemMetrics(SM_CYSCREEN);

				if ((finfo.biWidth > (screenwidth/3))&&(finfo.biHeight > (screenheight/3)))
					tile = FALSE;
			}
		}

		CloseHandle(fhnd);
	}
	return tile;
}

// Write the tile parameter to the registry, the 'only' way to control tiling and centering
void SetTileParameter(bool tile)
{
	HKEY key;

	if (RegOpenKeyEx(HKEY_CURRENT_USER,"Control Panel\\Desktop",0,KEY_SET_VALUE,&key)==ERROR_SUCCESS)
	{
		char data[8];

		itoa(tile,data,10);
		RegSetValueEx(key,"TileWallpaper",NULL,REG_SZ,(UCHAR*)data,8);
		RegCloseKey(key);
	}
}

// Actually set the wallpaper, checks it for a .bmp extension and that it exists
void SetWallpaper(const char *wallpaper, int tile)
{
	if (wallpaper)
	{
		char *ext = strrchr(wallpaper,'.');
		if (ext)
		{
			if (stricmp(ext,".bmp")==0)
			{
				struct stat status;
				if (stat(wallpaper,&status)!= -1)
				{
					if (tile==2)
					{
						SetTileParameter(Calculate(wallpaper));
					}
					else
					{
						SetTileParameter(bool(tile));
					}

					SystemParametersInfo(SPI_SETDESKWALLPAPER,0,(void*)wallpaper,SPIF_SENDWININICHANGE);
				}
				else
				{
					cout << "Unable to find file\n";
				}
			}
			else
			{
				cout << "File must have a .bmp extension\n";
			}
		}
		else
		{
			cout << "File must have a .bmp extension\n";
		}
	}
}

int Tile(const char *tile)
{
	int ret = 2;

	if (tile)
	{
		if (tile[0]=='t')
			ret = 1;
		else if (tile[0]=='c')
			ret = 0;
	}

	return ret;
}

void main(int argc, char *argv[])
{

	switch (argc)
	{
		case 1:
			cout << "usage: spaper <wallpaper.bmp> [<t> or <c>]\n";
			cout << "where 'wallpaper.bmp' is a valid bitmap, 't' is tile and 'c' is centre.\n";
			cout << "if you don't supply a 't' or 'c' the program will decide based on it's own algorithm.\n";
			break;

		case 2:
			SetWallpaper(argv[1],2);
			break;
		case 3:
			SetWallpaper(argv[1],Tile(argv[2]));
			break;
	}
}
